* =======================================================================
* INIT_ASM.
*
* This file is included by QLTdis_asm and is used to define various
* initialisation routines and parameters used by them. The jobs main
* initialisation and each loop initialisation are defined in this file.
* =======================================================================



* =======================================================================
* One off job initialisation :
* =======================================================================
* 1. Set job's dataspace pointers etc
* 2. Set mode 4 if not already set
* 3. Open title CON_ channel
* 4. Open output CON_ channel
* 5. Check errors - cannot continue if detected.
*
* Returns Z set if all ok.
* =======================================================================
job_init    adda.l   a6,a4              ; A4.L = start of dataspace

* -----------------------------------------------------------------------
* Set mode 4 if not already set. Do not change from TV to monitor or
* vice versa. We must preserve the display type if we reset the mode.
* -----------------------------------------------------------------------
            moveq    #mt_dmode,d0
            moveq    #-1,d1             ; Read current mode
            moveq    #-1,d2             ; Read current display type
            trap     #1                 ; Do it
            tst.l    d0                 ; Did it work ?
            bne.s    con_done           ; No, bale out

            tst.b    d1                 ; 0 in D1.B = Mode 4
            beq.s    mode_4             ; No need to set mode 4

            moveq    #mt_dmode,d0
            clr.l    d1                 ; We need mode 4
            trap     #1                 ; Set mode 4 (d2 = display type)
            tst.l    d0                 ; Check it
            bne.s    con_done           ; Bale out if errors detected

* -----------------------------------------------------------------------
* Mode 4 is current mode. Open the title window at the top of the screen.
* -----------------------------------------------------------------------
mode_4      lea      con_def,a1         ; Window definition
            movea.w  ut_con,a2          ; Utility to define a window etc
            jsr      (a2)               ; Do it
            tst.l    d0                 ; Did it work ok ?
            bne.s    con_done           ; No, exit routine
            move.l   a0,con_id(a4)      ; Store console id for title channel

* -----------------------------------------------------------------------
* Now open the output window underneath the title one.
* -----------------------------------------------------------------------
            lea      con_def2,a1        ; Output window definition
            movea.w  ut_con,a2          ; Utility again
            jsr      (a2)               ; Do it
            tst.l    d0                 ; Did it work ?
            bne.s    con_done           ; No, exit routine
            move.l   a0,con_id2(a4)     ; Store console id for output channel

            moveq    #0,d0              ; No errors detected
con_done    rts

*------------------------------------------------------------------------
* Definition for title window channel
*------------------------------------------------------------------------
con_def     dc.b    red                 ; Border colour
            dc.b    1                   ; Border width
            dc.b    white               ; Paper/strip colour
            dc.b    black               ; Ink colour
            dc.w    448                 ; Width
            dc.w    24                  ; Height
            dc.w    32                  ; Start position x
            dc.w    16                  ; Start position y


*------------------------------------------------------------------------
* Definition for output window channel
*------------------------------------------------------------------------
con_def2    dc.b    red                 ; Border colour
            dc.b    1                   ; Border width
            dc.b    white               ; Paper/strip colour
            dc.b    black               ; Ink colour
            dc.w    448                 ; Width
            dc.w    200                 ; Height
            dc.w    32                  ; X org
            dc.w    40                  ; Y org


* =======================================================================
* Once per disassembly initialisation :
* =======================================================================
* 1. Clear screen & show headings etc
* 2. Ask for start address, validate
* 3. Ask for end address, validate
* 4. Ask for 'printer' channel
* 5. Clear screen
* 6. Show details
*
* Returns Z set if no errors.
* If user quits, sets pc_addr(a4) = -1
* =======================================================================
loop_init   bsr      headings            ; Display headings etc in title window

*------------------------------------------------------------------------
* Clear screen & ask for start address
*------------------------------------------------------------------------
            movea.l  con_id2(a4),a0      ; output console id
            bsr      cls                 ; Clear screen
get_start   lea      start_addr,a1       ; Our prompt
            bsr      prompt              ; Print it
            bsr      input               ; Wait for user to type something
            tst.w    d1                  ; Anything typed ?
            bne.s    got_start           ; User typed something

            move.l   #-1,pc_addr(a4)     ; Store a 'user quit' address
            bra      init_done           ; Finished

*------------------------------------------------------------------------
* User has typed an address. Convert from a string into a long.
*------------------------------------------------------------------------
got_start   nop                          ; WE COME BACK TO THIS NEXT TIME

*------------------------------------------------------------------------
* Ask for end address - if we get here
*------------------------------------------------------------------------
get_end     lea      end_addr,a1         ; Our prompt
            bsr      prompt              ; Print it
            bsr      input               ; Wait for user input
            tst.w    d1                  ; Anything typed ?
            bne.s    got_end             ; User typed something

            move.l   #$ffffffff,pc_end(a4)  ; Very high end address
            bra      get_printer            ; Go for the printer name next
*------------------------------------------------------------------------
* User has typed an address. Convert from a string into a long.
*------------------------------------------------------------------------
got_end     nop                          ; WE COME BACK TO THIS NEXT TIME

*------------------------------------------------------------------------
* Ask for additional file/printer device
*------------------------------------------------------------------------
get_printer lea      print_dev,a1        ; Our prompt
            bsr      prompt              ; Print it
            bsr      input               ; Wait for user input

            lea      pr_dev,a0           ; Storage for printer name
            move.w   d1,(a0)+            ; Save filename length
            subq.w   #1,d1               ; Adjust for dbra loop

*------------------------------------------------------------------------
* At this point :
*
* A1.L = first character in input buffer for printer name
* A0.L = first character position in pr_dev buffer
* D1.W = Number of character in filename minus 1
*------------------------------------------------------------------------
move_prname move.b   (A1)+,(A0)+         ; Move a single character
            dbra     d1,move_prname      ; And the rest            

*------------------------------------------------------------------------
* Clear screen again - in case user typed silly stuff !
*------------------------------------------------------------------------
            movea.l  con_id2(a4),a0      ; Output console id
            bsr      cls                 ; Clear screen

*------------------------------------------------------------------------
* Show addresses and filenames again prior to start.
*------------------------------------------------------------------------
show_start  lea      start_addr,a1       ; Our prompt
            move.w   (a1),-(a7)          ; Stack string length
            move.w   #14,(a1)            ; Reduce string length
            bsr      prompt              ; Print it
            lea      start_addr,a1       ; A1 is trashed by 'prompt'
            move.w   (a7)+,(a1)          ; Restore original length

*------------------------------------------------------------------------
* Print start address here - WE WILL DEAL WITH THIS NEXT TIME
*------------------------------------------------------------------------
            nop
            bsr      line_feed           ; Print a new line

show_end    lea      end_addr,a1         ; Our prompt
            move.w   (a1),-(a7)          ; Save current string size
            move.w   #12,(a1)            ; New 'pretend' length
            bsr      prompt              ; Print it
            lea      end_addr,a1         ; A1 is trashed remember 
            move.w   (a7)+,(a1)          ; Restore original length

*------------------------------------------------------------------------
* Print end address here - WE WILL DEAL WITH THIS NEXT TIME
*------------------------------------------------------------------------
            nop
            bsr      line_feed           ; Print a new line

show_prname lea      print_dev,a1        ; Our prompt
            move.w   (a1),-(a7)          ; Store original string length
            move.w   #15,(a1)            ; New length
            bsr      prompt              ; Print it
            lea      print_dev,a1        ; Prompt trashes A1
            move.w   (a7)+,(a1)          ; Restore original length
            lea      pr_dev,a1           ; Printer device name
            bsr      prompt              ; Print printer filename
            bsr      line_feed           ; And a final linefeed
    
init_done   moveq    #0,d0               ; No errors
            rts


start_addr  dc.w    37
            dc.b    'Start address (ENTER only to quit) : '

end_addr    dc.w    39
            dc.b    'End address (ENTER only = until ESC) : '

print_dev   dc.w    32
            dc.b    'Printer device (ENTER = none) : '

pr_dev      ds.w    26                  ; 52 bytes for max file name & length
