*
* Enhanced version of the QL DATE function as written by Mark Knight.
*  Copyright Mark Knight 1999.
*
* First set up some useful constants.
*
DayLength  EQU         86400           86,400 = 1 day in seconds.
YearLength EQU         31536000        31,536,000 = 1 year in seconds.
WillBump   EQU         3035647         Maximum offset within 2029 before overflow.
HighLimit  EQU         2147483647      Maximum value of a 32-bit word (longword).
*
Start      MOVE.W      $110,A0         $110 = BP.INIT vector.
           LEA.L       Defines,A1      Load definition address into A1.
           JMP         (A0)            jump to BP.INIT it will return to caller.
*
OutRange   MOVEQ       #-4,D0          -4 = OUT OF RANGE error code.
           RTS
*
DudParam   MOVEQ       #-15,D0         -15 = BAD PARAMETER error code.
DudExit    RTS
*
* First part of main code checks to see if there are any parameters, fetches them
* if there are, else branch to other routine.
*
Main       CMPA.L      A3,A5           Any parameters?
           BEQ         NoParams        If not then work as old DATE function.
           MOVE.W      $118,A0         $118 = CA.GTLIN to fetch long integers.
           JSR         (A0)            Fetch the arguments.
           BMI.S       DudExit         If so report it.
           CMPI.W      #6,D3           Are there six of them?
           BNE.S       DudParam        If not then something is wrong.
*
*  OK we now have six longwords on the RI stack in order:
*  year, month, day, hours, minutes, seconds.
*
* First fetch the seconds and make sure they are in range.
*
           MOVE.L      $58(A6),A1      Fetch RI stack pointer.
           MOVE.L      20(A6,A1.L),D7  Fetch seconds into main accumulator.
           BMI.S       DudParam        If negative then report error.
           CMPI.L      #59,D7          Is it more than 59 seconds?
           BHI.S       OutRange        If so report error.
*
* Next we fetch minutes, multiply into seconds and add to total in D7.
*
           MOVE.L      16(A6,A1.L),D6  Fetch minutes.
           BMI.S       DudParam        If negative then report error.
           CMPI.L      #59,D6          Is it more than 59 minutes?
           BHI.S       OutRange        If so report error.
           MULU.W      #60,D6          Multiply by 60 to find second count.
           ADD.L       D6,D7           Add seconds to main accumulator.
*
* Now fetch hours, multiply into seconds and add to total in D7.
*
           MOVE.L      12(A6,A1.L),D6  Fetch hours.
           BMI.S       DudParam        If negative then report error.
           CMPI.L      #23,D6          Is it more than 23 hours?
           BHI.S       OutRange        If so report error.
           MULU.W      #3600,D6        Multiply by 3,600 to find second count.
           ADD.L       D6,D7           Add to main accumulator.
           MOVE.L      D7,D1           Make copy of seconds count for later check.
*
* Fetch date.  Sort out if it's too big later.
*
           MOVE.L      8(A6,A1.L),D6   Fetch date.
           SUBQ.L      #1,D6           Subtract 1 for use as offset/multiplier.
           BMI.S       DudParam        If negative then report error.
*
* Fetch month and use lookup table to find offset in seconds from start of year.
* Note that for now we assume it's not a leap year.
*
           MOVE.L      4(A6,A1.L),D5   Fetch month.
           SUBQ.L      #1,D5           Subtract one for lookup table.
           BMI.S       DudParam        If result is negative report error.
           CMPI.L      #11,D5          Is month bigger than 11?
           BHI.S       OutRange        If so then month is too big.
           LEA.L       MonthTab,A2     Put address of lookup table in A2.
           MOVE.L      D5,D2           Make temporary copy of month.
           MULU.W      #4,D2           Multiply by 4 for use as pointer.
           ADDA.L      D2,A2           Add to base of table.
           ADD.L       (A2),D7         Add value from table to main accumulator.
*
* Fetch year, check it is in range and calculate the year offset for multiplier.
*
           MOVE.L      (A6,A1.L),D4    Fetch year.
           ADDA.L      #24,A1          Clear RI stack altogether.
           MOVE.L      A1,$58(A6)      Store RI stack pointer.
           MOVE.L      D4,D3           Make copy of year.
           SUBI.L      #1961,D3        Subtract 1961, first available year.
           BMI         DudParam        If result is negative report error.
           CMPI.L      #136,D3         Is result greater than 136?
           BHI         OutRange        If so then year is too big.
           BCS.S       TestLeap        If less than 136 then skip extra checks.
*
* OK year 2097 specified so check if the rest of parameters are in range.
*
           CMPI.L      #1,D5           Check if month is 1 (0 = Jan 1 = Feb).
           BHI.S       OverFlow        If later then report error.
           BCS.S       LeapDone        If earlier then continue.
           CMPI.L      #5,D6           Is it 6 Feb or later?
           BHI.S       OverFlow        If later then report error.
*
* Remember we subtracted 1 from date so 6th of month is stored as 5.
*
           BCS.S       LeapDone        If earlier then continue.
           CMPI.L      #23295,D1       it's 6 Feb so check seconds count.
*
* 23,295 seconds = 06:28:15 which is the highest valid second count from the
* start of 2097 Feb 06.
*
           BHI.S       OverFlow        If too late in the day then report error.
           BRA.S       LeapDone        2097 is not a leap year so skip checks.
*
OverFlow   MOVEQ       #-18,D0         -18 = OVERFLOW error code.
           RTS
*
* Now year is in D4 and year offset is in D3 and we know year is in range.
*
* Following code checks to see if the specified year is a leap year.
* If it is then we check to see if it is later than 29 Feb and if so add a day.
* If it's February then a check for the date being out of range is done.
*
TestLeap   BTST        #0,D4           Is year odd?
           BNE.S       LeapDone        If so it can't be a leap year.
           BTST        #1,D4           Is year odd multiple of 2?
           BNE.S       LeapDone        If so it can't be a leap year.
           CMPI.L      #1,D5           Is it March or later? (Jan=0, Dec=11)
           BHI.S       March           If March or later then move on.
           BEQ.S       February        If it is February go sort it out.
           BRA.S       LeapDone        If we reach here it must be January.
*
February   CMPI.L      #28,D6          Is date minus 1 more than 28?
           BHI.S       BadRange        If so report error.
           MOVE.L      D6,D2           Make temporary copy of date.
           BEQ.S       DateDone        If it's zero then no work to do.
           SUBQ.L      #1,D2           Subtract 1 for DBRA counter.
           BRA.S       DateLoop        Go add the new factor.
*
March      ADDI.L      #DayLength,D7   Add the length of day to main accumulator.
*
* OK leap year details sorted so check the date using the lookup table.
*
LeapDone   LEA.L       MonthLens,A2    Put address of lookup table in A2.
           MOVE.L      D5,D2           Make temporary copy of month.
           MULU.W      #4,D2           Multiply by 4 for use as pointer.
           ADDA.L      D2,A2           Add to base of table.
           MOVE.L      (A2),D1         Fetch maximum date for this month.
           CMP.L       D1,D6           Is date bigger?
           BHI.S       BadRange        If so then report error.
           MOVE.L      D6,D2           Make temporary copy of date.
           BEQ.S       DateDone        If it's zero then no work to do.
           SUBQ.L      #1,D2           Subtract 1 for DBRA counter.
DateLoop   ADD.L       #DayLength,D7   Add the daylength to main accumulator.
           DBRA        D2,DateLoop     Add it until days exhausted.
*
* Calculate how many leap years have already passed, add extra day for each.
*
DateDone   MOVE.L      D3,D2           Copy year offset for division.
           ASR.L       #2,D2           Divide by 4 discarding remainder.
           BEQ.S       TryYear         If result is zero we are done here.
           SUBQ.L      #1,D2           Subtract 1 from leap years for DBRA.
LeapLoop   ADD.L       #DayLength,D7   Add length of a day for each leap year.
           DBRA        D2,LeapLoop     If more years left then loop back.
*
* Calculate how many years have passed and add a year in seconds for each.
*
TryYear    MOVEQ.L     #0,D1           Zero the temporary accumulator.
           CMPI.L      #68,D3          Check year offset against 2029 (1961 + 68)
           BLT.S       CheckDone       If it is lower then no work to be done.
           SUB.L       #WillBump,D7    Reduce main accumulator by suitable margin.
           MOVE.L      #HighLimit,D1   Set other accumulator to overflow value.
           SUB.L       #68,D3          Reduce year count for later loop.
CheckDone  MOVE.L      D3,D2           Copy year offset again.
           BEQ.S       YearDone        If it's zero no more work to do.
           SUBQ.L      #1,D2           Subtract 1 from years for DBRA...
YearLoop   ADD.L       #YearLength,D1  ...add length of one year in seconds.
           DBRA        D2,YearLoop     Loop back if years not finished.
*
YearDone   ADD.L       D7,D1           Whole years done, add offset within year.
           BRA.S       LongRet         Go and return it.
*
BadParam   MOVEQ       #-15,D0         -15 = BAD PARAMETER error code.
BadExit    RTS
*
BadRange   MOVEQ       #-4,D0          -4 = OUT OF RANGE error code.
           RTS
*
* Following short routine reads the time from the QL clock, then falls through
* into routine to turn it into a floating point number from a 32-bit longword.
*
NoParams   MOVE.L      #$13,D0         Select MT.RCLCK routine.
           TRAP        #1              Call it.
*
* How to return a SIGNED LONGWORD value, coerced into FLOATING POINT,
* To SuperBASIC from a FUNCTION extension.  From Simon Goodwin's DIY Toolkit.
*
* At the start of the code, the LONGWORD value is in data register D1.
*
LongRet    MOVE.W      D1,D4           D4 will be exponent.
           MOVE.L      D1,D5           D5 will be mantissa.
           BEQ.S       Norlised        Zero is a trivial case.
           MOVE.W      #2079,D4        First guess at exponent.
           ADD.L       D1,D1           Is it normalised?
           BVS.S       Norlised        If so, return it.
           SUBQ.W      #1,D4           Not ready, halve exponent.
           MOVE.L      D1,D5           Doubled mantissa moved to D5.
           MOVEQ       #16,D0          Try 16 bit shift first.
*
Normalise  MOVE.L      D5,D1           Make copy of mantissa.
           ASL.L       D0,D1           Shift mantissa D0 places left.
           BVS.S       Too_Far         Try next shift distance.
           SUB.W       D0,D4           correct exponent for that shift.
           MOVE.L      D1,D5           New mantissa is closer.
Too_Far    ASR.W       #1,D0           Half the shift distance for next try.
           BNE.S       Normalise       Half the previous shift each time.
*
* Make sure there is enough space for the result.
*
Norlised   MOVEQ       #6,D1           Number of bytes needed is 6.
           MOVE.W      $11A,A0         Select BV.CHRIX to reserve space.
           JSR         (A0)            Call the routine
           MOVE.L      $58(A6),A1      Fetch safe A1 value.
           SUBQ.L      #6,A1           Point to the new value.
           MOVE.L      A1,$58(A6)      update stack pointer.
           MOVE.L      D5,2(A1,A6.L)   Stack the mantissa.
           MOVE.W      D4,0(A1,A6.L)   Stack the exponent.
           MOVEQ       #2,D4           Indicate floating point result.
           MOVEQ       #0,D0           Indicate no error.
           RTS
*
* Table of offsets for non-leap year, in seconds from start of year.
* Using this method keeps code simpler.
*
MonthTab   DC.L        0               January 1st 00:00:00
           DC.L        2678400         February 1st 00:00:00
           DC.L        5097600         March 1st 00:00:00
           DC.L        7776000         April 1st 00:00:00
           DC.L        10368000        May 1st 00:00:00
           DC.L        13046400        June 1st 00:00:00
           DC.L        15638400        July 1st 00:00:00
           DC.L        18316800        August 1st 00:00:00
           DC.L        20995200        September 1st 00:00:00
           DC.L        23587200        October 1st 00:00:00
           DC.L        26265600        November 1st 00:00:00
           DC.L        28857600        December 1st 00:00:00
*
* Lookup TABLE of lengths-1 of months, in days.  Once again keeps code simpler.
*
MonthLens  DC.L        30              January
           DC.L        27              February
           DC.L        30              March
           DC.L        29              April
           DC.L        30              May
           DC.L        29              June
           DC.L        30              July
           DC.L        30              August
           DC.L        29              September
           DC.L        30              October
           DC.L        29              November
           DC.L        30              December
*
Defines    DC.W        0               No PROCedures
           DC.W        0               End of PROCedure list.
           DC.W        1               One FuNction
           DC.W        Main-*          Position of code for FuNction
           DC.B        4               Length of name
           DC.B        'DATE'          Characters of name
           DC.W        0               End of FuNction list
           END
