* =======================================================================
* QLTdis_asm
* =======================================================================
* A simple disassembler program for the 68008 instruction set.
* 
* Written for QL Today using George Gwilt's GWASL assembler. 
* Thanks to George for his free donation of this assembler.
*
* QLTdis is copyright - Norman Dunbar 1998/1999
* However, permission is given for unlimited use and abuse by whoever
* provided that the original author (me) gets some credit !
*
* -----------------------------------------------------------------------
* WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
* -----------------------------------------------------------------------
* GWASL does not like TAB characters in its source files. Make sure that
* your chosen editor is set to convert TAB into a number of spaces (I use
* 4 spaces) AND/OR that you simply type spaces instead of TAB to line up 
* columns. This is VERY IMPORTANT.
* -----------------------------------------------------------------------
* WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
* -----------------------------------------------------------------------

            in win1_source_QLTdis_equates_asm

* =======================================================================
* CODE STARTS HERE.
* First part of any job file is a standard QDOS Job header. When EXEC'd
* the first instruction executed will be the one at 'start' which is a
* 2 byte short branch.
*
* QDOS JOB HEADER FORMAT :
*
* 6 bytes starting on an even address (most assemblers handle this for
* you). These 6 bytes are usually a short branch instruction followed by
* 4 bytes of padding (as in the following) or a JMP.L label.
* The id word $4AFB - this indicates a QDOS job to the system.
* A single word holding the size of the job name in bytes.
* The actual job's name.
* =======================================================================
start       bra.s   qltdis              ; 2 bytes short jump
            dc.l    0                   ; 4 bytes padding
            dc.w    $4afb               ; Job identifier must be at location 6 
            dc.w    6                   ; 6 bytes in job's name
            dc.b    'QLTdis'            ; 6 bytes of job's name


* =======================================================================
* The job's code proper starts here. This is the 'main control' part and 
* defines the program's operation from a top down viewpoint.
*
* On entry here, the following registers are set up :
*
* A6.L = Base address of the Job's header ie, the address of 'start'.
* (A6,A4.L) = Bottom of the job's dataspace area - the first usable byte.
* (A6,A5.L) = Top of job's dataspace area - the last usable byte.
* A7 = The job's USP (User Stack Pointer)
*
* There may also be data on the job's stack, but we are ignoring it here
* and will discuss it later when examining QDOS.
* =======================================================================
qltdis      bsr     job_init            ; Once only initialisation
            bne.s   exit                ; Errors - cannot continue

main_loop   bsr     loop_init           ; Once per disassembly initialisation
            bne.s   exit                ; Errors - cannot continue
            cmpi.l  #-1,pc_addr(a4)     ; Has user quit ?
            beq.s   exit                ; Yes - all done

dis_loop    bsr     diss                ; Disassemble one instruction
            bsr     print               ; Print disassembly
            bsr     escape              ; Check ESC or done (Z = carry on)
            beq.s   dis_loop            ; Not done - do some more


* =======================================================================
* User pressed ESC or we are past the stop address. Terminate this loop by
* closing the 'printer' file etc. Then go back and ask the user for more.
* =======================================================================
main_end    bsr     loop_term           ; Once per disassembly termination
            bra.s   main_loop           ; Prepare for another disassembly


* =======================================================================
* The program is done or a massive error has occurred. Bale out.
* =======================================================================
exit        bra     all_done            ; We are finished.


* =======================================================================
* This is the end of the 'main control' part of the program. Following on
* from here are the various routines used by the above.
* =======================================================================

            in  win1_source_QLTdis_init_asm
            in  win1_source_QLTdis_diss_asm
            in  win1_source_QLTdis_term_asm

* =======================================================================
* And following on from here is the various utility sub-routines that are
* used from any part of the above code.
* =======================================================================

            in  win1_source_QLTdis_utils_asm
