* =======================================================================
* UTILS_ASM
* 
* This file is included by QLTdis_asm and defines a large number of sub-
* routines that are called from various parts of the program. These are
* called more tahn once and so have been defined as sub-routines to avoid
* duplicating code. In addition, any errors will only need to be fixed
* here once.
* =======================================================================



* =======================================================================
* Are we done yet routine(s)
* =======================================================================
* 1. Is new current address >= stop address ?
* 2. Has ESC been pressed ?
*
* Returns Z set if not done and if ESC not pressed.
* =======================================================================
escape      moveq    #1,d0                    ; ESC pressed
            rts


* =======================================================================
* Headings :
* =======================================================================
* 1. Clear title window
* 2. Print titles
* =======================================================================
headings    movea.l  con_id(a4),a0       ; Title channel id
            bsr.s    cls                 ; Clear screen
            lea      mes_title,a1        ; Title string
            bsr.s    prompt              ; Print title string
            rts

mes_title   dc.w     55
            dc.b     'QLTdis - copyright Norman Dunbar 1998/1999',linefeed
            dc.b     'Version 1.00'


* =======================================================================
* CLS :
* =======================================================================
* 1. Clear the (screen) channel whose id is in A0.
* =======================================================================
cls         moveq    #sd_clear,d0        ; CLS
            moveq    #infinite,d3        ; Infinite timeout
            trap     #3                  ; CLS title window
            rts


* =======================================================================
* Prompt :
* =======================================================================
* 1. Print the string at (A1) to the channel in A0.
*
* Z set if all ok, unset if not.
* =======================================================================
prompt      movea.w ut_mtext,a2          ; Print a string utility
            jsr     (a2)                 ; Print it
            tst.l   d0                   ; Check for errors
            rts


* =======================================================================
* Line_Feed :
* =======================================================================
* 1. Print a linefeed to the channel in A0.
*
* Z set if all ok, unset if not.
* =======================================================================
line_feed   movem.l d1/a1,-(a7)          ; Preserve any registers reqd
            moveq   #io_sbyte,d0         ; Send one byte to channel
            moveq   #linefeed,d1         ; Byte to send = linefeed
            moveq   #infinite,d3         ; Timeout
            trap    #3                   ; Do it
            movem.l (a7)+,d1/a1          ; Restore
            tst.l   d0                   ; Set Z if errors
            rts

* =======================================================================
* Input :
* =======================================================================
* 1. Wait for user input from the channel id in A0.
*
* Returns the input length (not counting the ENTER character) in D1.W
* Returns the address of the first character in the buffer in A1.L
* Preserves the channel id in A0.L
* Z set if all ok, unset if not.
* =======================================================================
input       lea     buffer+2,a1         ; Our buffer address plus 2
            move.l  a1,-(a7)            ; Save it on the stack
            moveq   #io_fline,d0        ; Input some bytes (inc linefeed)
            moveq   #60,d2              ; Buffer size maximum
            moveq   #infinite,d3        ; Inifinite timeout
            trap    #3

            move.l  (a7)+,a1            ; Restore buffer pointer
            subq.w  #1,d1               ; Subtract the linefeed character
            move.w  d1,-2(a1)           ; Store length in buffer
            tst.l   d0                  ; Did it all work ?
            rts

 
buffer      ds.w    16                  ; 60 chars for input plus 1 word for size.
